/* This file is part of the KDE project
   Copyright (C) 2004-2017 David Faure <faure@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/
#include <KDirModel>
#include <KDirLister>

#include <QTest>
#include <QSignalSpy>
#include <QObject>

class TestKIOFonts : public QObject
{
    Q_OBJECT
private slots:
    void testDirLister();
};

void TestKIOFonts::testDirLister() // #160057
{
    // GIVEN
    QUrl url(QStringLiteral("fonts://foo/System"));
    KDirModel dirModel;
    KDirLister *dirLister = dirModel.dirLister();
    dirLister->setAutoErrorHandlingEnabled(false, nullptr);

    // WHEN
    dirLister->openUrl(url, KDirLister::NoFlags);

    // THEN
    QSignalSpy completedSpy(dirLister, SIGNAL(completed()));
    QVERIFY(completedSpy.isValid());
    QVERIFY(completedSpy.wait());

    QCOMPARE(dirLister->url().toString(), QString("fonts:/System"));

}

QTEST_MAIN(TestKIOFonts)
#include "test_kio_fonts.moc"

